use {Message, Channel, User};

use std::sync::Arc;
use std::collections::btree_set::BTreeSet;

#[derive(Debug)]
pub enum Event {
    Message { message: Message },     // A user sends a message

    SelfJoin { channel: Channel },    // We join a channel
    SelfQuit { channel: Channel },    // We quit a channel

    UserJoin { channel: Channel, user: User }, // A user joins a channel
    UserQuit { channel: Channel, user: User }  // A user quits a channel
}

#[derive(Debug)]
pub struct Envelope {
    pub from: Option<String>,
    pub event: Event,
    pub to: Vec<String>
}

impl Envelope {
    pub fn new (event: Event) -> Envelope {
        Envelope {
            from: None,
            event: event,
            to: vec![]
        }
    }
}
