use std::env;
use std::fs::File;
use std::io::Read;

extern crate tenquestionmarks;
use tenquestionmarks::Tenquestionmarks;

extern crate toml;
use toml::Parser;

#[macro_use]
extern crate log;
extern crate env_logger;

extern crate time;

fn init_logger () {
    let mut builder = env_logger::LogBuilder::new();
    builder.format(|record: &log::LogRecord| {
        let t = time::now();
        format!("{} {}:{}: {}", time::strftime("%Y-%m-%d %H:%M:%S", &t).unwrap(), record.level(), record.location().module_path(), record.args())
    }).filter(None, log::LogLevelFilter::Info);;

    if env::var("RUST_LOG").is_ok() {
       builder.parse(&env::var("RUST_LOG").unwrap());
    }

    builder.init().unwrap();
}

fn main () {
    init_logger();

    let config_file_name = env::args().nth(1).unwrap_or("tenquestionmarks.toml".into());
    match File::open(&config_file_name) {
        Ok(mut file) => {
            let mut contents = String::new();
            match file.read_to_string(&mut contents) {
                Ok(_) => {
                    let mut parser = Parser::new(&contents);
                    match parser.parse() {
                        Some(configuration) => {
                            info!("Loaded configuration from: {}", config_file_name);
                            match Tenquestionmarks::from_configuration(configuration) {
                                Ok(tqm) => {
                                    info!("tenquestionmarks initialized successfully");
                                    tqm.run();
                                },
                                Err(e) => error!("Failed to initialize tenquestionmarks: {:?}", e)
                            }
                        },
                        None => {
                            error!("Failed to parse config file {}: {:?}. Config file must be a valid TOML file.", config_file_name, parser.errors);
                        }
                    }
                },
                Err(e) => {
                    error!("Failed to open config file {}: {:?}", config_file_name, e);
                }
            }
        },
        Err(_) => {
            error!("Failed to open config file! Please specify path to a config file.");
        }
    }
}
