use modules::{Module, EventLoop};
use toml::Table;

use std::sync::Arc;
use std::sync::mpsc::Receiver;
use transformable_channels::mpsc::ExtSender;

use helpers::command::split_command;
use event::{Event, Envelope};

pub struct EchoModule {
    prefix: String
}

impl EchoModule {
    pub fn new (_: &Table, configuration: &Table) -> Module {
        let prefix = configuration.get("prefix")
                                  .and_then(|value| value.as_str())
                                  .unwrap_or("!echo");

        Module {
            module_type: String::from("echo"),
            event_loop: Box::new(EchoModule {
                prefix: String::from(prefix)
            }),
            config: configuration.clone()
        }
    }
}

impl EventLoop for EchoModule {
    fn run(&self, _: Box<ExtSender<Envelope>>, receiver: Receiver<Arc<Envelope>>) {
        loop {
            match receiver.recv() {
                Ok(envelope) => {
                    match envelope.event {
                        Event::Message { ref message } => {
                            debug!("Received message from module {:?}... {:?}", envelope.from, message.content);
                            match split_command(&message.content) {
                                Some((command, argument)) => {
                                    if command == self.prefix {
                                        message.reply(argument);
                                    }
                                },
                                _ => {}
                            }
                        }
                        _ => ()
                    }
                }
                Err(error) => { error!("Error {:?}", error) }
            }
        }
    }
}
